////////////////////////////////////////////////////////////
//
// LOGIN / LOGOUT
//
////////////////////////////////////////////////////////////

function api_getUser(onreply,host) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            if (req.status == 200) {
                onreply(JSON.parse(req.responseText));
            } else {
                onreply(null);
            }
        }
    };

    var url = "/users/api/whoami";
    if (host) url = host+url;

    req.open("GET", url, true);
}

function api_async_getUser(host) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = () => {
            if (req.readyState == 4) {
                if( req.status == 204) {
                    resolve(null);
                }
                else if (req.status == 200) {
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }
                } else {
                    reject("Status = " + req.status);
                }
            }
        };
        var url = "/users/api/whoami";
        if (host) url = host+url;

        req.open("GET", url, true);
        req.send();
    });
}

function api_login(username,password,onreply,host) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            if (req.status == 200) {
                onreply(JSON.parse(req.responseText));
            } else {
                onreply(null);
            }
        }
    };

    var url = "/users/api/login";
    if (host) url = host+url;
    req.open("POST", url, true);

    req.send(JSON.stringify({"Username":username,"Password":password}));
}

async function api_async_login(username,password,host) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
                if (req.status == 200) {
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }
                } else {
                    reject("Status = " + req.status);
                }
            }
        };

        var url = "/users/api/login";
        if (host) url = host+url;
        req.open("POST", url, true);

        req.send(JSON.stringify({"Username":username,"Password":password}));
    });
}

function api_logout(onreply,host) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            //console.log("logout: status = "+req.status);
            if (req.status == 200 || req.status == 204) {
                onreply(true);
            } else {
                onreply(false);
            }
        }
    };

    var url = "/users/api/logout";
    if (host) url = host+url;
    req.open("GET", url, true);
    req.send();
}

async function api_async_logout(host) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
                if (req.status == 200 || req.status == 204) {
                    resolve(req.status);
                } else {
                    reject("Status = " + req.status);
                }
            }
        };

        var url = "/users/api/logout";
        if (host) url = host+url;
        req.open("GET", url, true);
        req.send();
    });
}

////////////////////////////////////////////////////////////
//
// ACCESS TOKENS
//
////////////////////////////////////////////////////////////

// data: { Userid : string,
//         Permissions : optional list([name : string, onoff : bool]),
//         Description : optional string }
// onreply: function(data : TokenData)
//    TokenData : {
//       Serial      : int
//       Name        : string
//       Owner       : string
//       Expires     : int
//       Description : string
//       Permissions : map( permname : string -> bool )
//    }
function api_createToken(data,host,onreply) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            switch (req.status) {
            case 200:
                onreply(JSON.parse(req.responseText));
                break;
            default:
                onreply();
                break;
            }

        }
    };
    var url = "/users/api/token/create";
    if (host) url = host+url;
    req.open("POST", url, true);
    req.send(JSON.stringify(data));
}

async function api_async_createToken(data,host) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
                switch (req.status) {
                case 200:
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }
                    break;
                default:
                    reject("Status = "+req.status);
                    break;
                }

            }
        };
        var url = "/users/api/token/create";
        if (host) url = host+url;
        req.open("POST", url, true);
        req.send(JSON.stringify(data));
    });
}

// serial : int
// onreply : function(ok : bool)
function api_deleteToken(serial,onreply,host) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4)
            switch (req.status) {
            case 200: onreply(true); break
            case 204: onreply(true); break;
            default:  onreply(false); break;
	    }
    }

    var url = "/users/api/token/delete?token-serial="+serial;
    if (host) url = host+url;

    req.open("GET", url, true);
    req.send();
}

async function api_async_deleteToken(serial,host) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4)
                switch (req.status) {
                case 200: resolve(req.status); break
                case 204: resolve(req.status); break;
                default:  reject("Status = "+onreply(true)); break;
	        }
        }

        var url = "/users/api/token/delete?token-serial="+serial;
        if (host) url = host+url;

        req.open("GET", url, true);
        req.send();
    });
}

// onreply: function(ok : bool, data : list(TokenData))
//    TokenData : {
//       Serial       : int,
//       Owner        : string,
//       Expires      : int,
//       Permissions  : map( permname : string -> bool ),
//       Description  : string
//    }
// user : undefined | username
function api_listTokens(onreply,user,host) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            //console.log("listTokens: status="+req.status,req.responseText)
            switch (req.status) {
            case 200: onreply(true,JSON.parse(req.responseText)); break;
            case 204: onreply(true,[]); break;
            default: onreply(false,[]); break;
            }
        }
    }

    var url = "/users/api/token/list" + (user !== undefined ? "?owner="+user : "");
    if (host) url = host+url;
    req.open("GET", url, true);
    req.send();
}

async function api_async_listTokens(user,host) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
                //console.log("listTokens: status="+req.status,req.responseText)
                switch (req.status) {
                case 200:
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }
                    break;
                default: reject("Status = "+status); break;
                }
            }
        }

        var url = "/users/api/token/list" + (user !== undefined ? "?owner="+user : "");
        if (host) url = host+url;
        //console.log("url = ",url);
        req.open("GET", url, true);
        req.send();
    });
}

////////////////////////////////////////////////////////////
//
// JOBS
//
////////////////////////////////////////////////////////////

function api_listJobs(onreply,user) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            if (req.status == 200 || req.status == 204) {
                var resp = JSON.parse(req.responseText);
                onreply(resp);
            }
        }
    };

    if (user !== undefined) {
        req.open("POST", "/users/api/jobs/list", true);
        req.send(JSON.stringify({"Owner" : user.Userid }));
    } else {
        req.open("GET", "/users/api/jobs/list", true);
        req.send();
    }
}

async function api_async_listJobs(user) {
    return await new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
                if (req.status == 200 || req.status == 204) {
                    try {
                        resolve(JSON.parse(req.responseText))
                    }
                    catch (err) {
                        reject(err);
                    }
                }
                else {
                    reject("Status = "+req.status);
                }
            }
        };

        let url = "/users/api/jobs/list" + (user !== undefined ? "?owner="+user : "");
        if (host) url = host+url;

        if (user !== undefined) {
            req.open("POST", url, true);
            req.send(JSON.stringify({"Owner" : user.Userid }));
        } else {
            req.open("GET", url, true);
            req.send();
        }

    });


    return res;
}

function api_startJob(token, onreply) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
	    if (onreply)
		switch (req.status) {
		case 200: onreply(true); break;
		case 204: onreply(true); break;
		default:  onreply(false); break;
		}

        }
    };
    req.open("GET", "/api/solve?token="+token, true);
    req.send();
}

async function api_async_startJob(token) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
		switch (req.status) {
		case 200: resolve(req.status); break;
		case 204: resolve(req.status); break;
		default:  reject("Status = "+req.status); break;
		}
            }
        };
        let url = "/api/solve?token="+token;
        if (host) url = host+url;
        req.open("GET",url,true);
        req.send();
    });
}

function api_deleteJobs(tokens, onreply) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
	    if (onreply)
		switch (req.status) {
		case 200: onreply(JSON.parse(req.responseText)); break;
		case 204: onreply([]); break;
		default:  onreply(); break;
		}
        }
    };
    req.open("POST", "/users/api/jobs/delete", true);
    var data = [];
    for (var i = 0; i < tokens.length; ++i) data[data.length] = String(tokens[i]);

    req.setRequestHeader("Content-Type","application/json");
    req.send(JSON.stringify(data));
}

async function api_async_deleteJobs(tokens) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
		switch (req.status) {
		case 200:
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }

                    break;
		case 204: resolve([]); break;
		default:  reject("Status = "+req.status); break;
		}
            }
        };
        let url = "/users/api/jobs/delete";
        if (host) url = host+url;
        req.open("POST", url, true);
        var data = [];
        for (var i = 0; i < tokens.length; ++i) data[data.length] = String(tokens[i]);

        req.setRequestHeader("Content-Type","application/json");
        req.send(JSON.stringify(data));
    });
}

function api_jobInfo(token, onreply) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
	    if (onreply)
		switch (req.status) {
		case 200: onreply(JSON.parse(req.responseText)); break;
		default:  onreply(); break;
		}

        }
    };
    req.open("GET", "/users/api/jobs/info?token="+token, true);
    req.send();
}

async function api_async_jobInfo(token) {
    return new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
	        if (onreply)
		    switch (req.status) {
		    case 200:
                        try {
                            resolve(JSON.parse(req.responseText));
                        }
                        catch (err) {
                            reject(err);
                        }
                        break;
		    default: reject("Status = "+req.status); break;
		    }

            }
        };
        let url = "/users/api/jobs/info?token="+token;
        if (host) url = host+url;
        req.open("GET", url, true);
        req.send();
    });
}

////////////////////////////////////////////////////////////
//
// USERS
//
////////////////////////////////////////////////////////////

function api_createUser(data,
                        onreply,
                        host) {
    var req = new XMLHttpRequest();

    req.onreadystatechange = function() {
        if (req.readyState == 4) {
            //console.log("createUser: readyState="+req.readyState+", status="+req.status);
            if (onreply)
                switch (req.status) {
                case 200: onreply(JSON.parse(req.responseText)); break;
                default:  onreply(false); break;
                }
        }
    };

    var body = {};
    body['Userid'] = data['Userid']+'';
    if (data['Name'])  body['Name']  = data['Name']+'';
    if (data['Email']) body['Email'] = data['Email']+'';
    if (data['Password']) body['Password'] = data['Password']+'';
    if (data['Permissions']) {
        permissions = data['Permissions'];
        var perms = [];
        for (var i = 0; i < permissions.length; ++i) {
            perms.push(permissions[i]+'');
        }
        body['Permissions'] = perms;
    }

    var url = "/users/api/user/create";
    if (host) url = host+url;
    req.open("POST", url, true);
    req.send(JSON.stringify(body));
}

async function api_async_createUser(data,host) {
    return await new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
                switch (req.status) {
                case 200:
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err)
                    }
                    break;
                default:
                    reject("Status = "+req.status);
                    break;
                }
            }
        };

        var body = {};
        body['Userid'] = data['Userid']+'';
        if (data['Name'])  body['Name']  = data['Name']+'';
        if (data['Email']) body['Email'] = data['Email']+'';
        if (data['Password']) body['Password'] = data['Password']+'';
        if (data['Permissions']) {
            permissions = data['Permissions'];
            var perms = [];
            for (var i = 0; i < permissions.length; ++i) {
                perms.push(permissions[i]+'');
            }
            body['Permissions'] = perms;
        }

        var url = "/users/api/user/create";
        if (host) url = host+url;
        req.open("POST", url, true);
        req.send(JSON.stringify(body));
    });
}

function api_deleteUser(userid,onreply) {
    var req = new XMLHttpRequest();

    req.onreadystatechange = function() {
        if (req.readyState == 4) {
	    if (onreply)
		switch (req.status) {
		case 200: onreply(true);  break;
		case 204: onreply(true);  break;
		default:  onreply(false); break;
		}
        }
    };
    req.open("GET", "/users/api/user/delete?userid="+userid, true);
    req.send();
}

async function api_async_deleteUser(userid,host) {
    var req = new XMLHttpRequest();

    let res = new Promise((resolve,reject) => {
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
		switch (req.status) {
		case 200: resolve(req.status); break;
		case 204: resolve(req.status); break;
		default:  reject("Status = "+req.status); break;
		}
            }
        };
    });
    let url = "/users/api/user/delete?userid="+userid;
    if (host) url = host+url;

    req.open("GET", url, true);
    req.send();

    return res;
}

function api_updateUser(data,onreply) {
    var req = new XMLHttpRequest();

    req.onreadystatechange = function() {
        if (req.readyState == 4) {
	    if (onreply)
		switch (req.status) {
		case 200: onreply(JSON.parse(req.responseText));  break;
		default:  onreply(); break;
		}
        }
    };
    req.open("POST", "/users/api/user/update", true);
    req.send(JSON.stringify(data));
}

async function api_async_updateUser(userdata,host) {
    let data = {}
    for (var k in userdata) {
        if      (k == 'Userid' ||
                 k == 'Name' ||
                 k == 'Email' ||
                 k == 'Password')
            data[k] = userdata[k];
        else if (k == 'Permissions') {
            //console.log('userperms = ',k)
            let userperms = data[k];
            let perms = [];
            for (var p in userperms) {
                if (p == "admin" ||
                    p == "submit" ||
                    p == "login" ||
                    p == "use-tokens" ||
                    p == "create-tokens" ||
                    p == "use-api")
                    perms.push([p,true]);
                else
                    throw new Error("Invalid permission");
            }
            data['Permissions'] = perms;
        }
        else
            throw new Error("Invalid user data entry '"+k+"'");
    }
    //console.log(userdata,data)

    return await new Promise((resolve,reject) => {
        var req = new XMLHttpRequest();
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
		switch (req.status) {
		case 200:
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }
                    break;
		default:
                    reject("Status = "+req.status); break;
		}
            }
        };
        let url = "/users/api/user/update";
        if (host) url = host+url;
        req.open("POST", url, true);
        req.send(JSON.stringify(userdata));
    });
}

function api_listUsers(onreply,host) {
    var req = new XMLHttpRequest();
    req.onreadystatechange = function() {
        if (req.readyState == 4) {
	        switch (req.status) {
            case 200: onreply(JSON.parse(req.responseText));  break;
	        default:  onreply(); break;
            }
        }
    };

    var url = "/users/api/user/list";
    if (host) url = host+url;
    req.open("GET", url, true);
    req.send();

}

async function api_async_listUsers(host) {
    var req = new XMLHttpRequest();
    let res = new Promise((resolve,reject) => {
        req.onreadystatechange = function() {
            if (req.readyState == 4) {
	        switch (req.status) {
                case 200:
                    try {
                        resolve(JSON.parse(req.responseText));
                    }
                    catch (err) {
                        reject(err);
                    }

                    break;
	        default:  reject("Status = "+req.status); break;
                }
            }
        };
    });

    var url = "/users/api/user/list";
    if (host) url = host+url;
    req.open("GET", url, true);
    req.send();
    return res;
}
